@@ -177,8 +177,9 @@ gupnp_control_point_dispose (GObject *object)
                 context = gupnp_control_point_get_context (control_point);
                 session = _gupnp_context_get_session (context);
 
-                soup_message_set_status (data->message, SOUP_STATUS_CANCELLED);
-                soup_session_cancel_message (session, data->message);
+                soup_session_cancel_message (session,
+                                             data->message,
+                                             SOUP_STATUS_CANCELLED);
 
                 get_description_url_data_free (data);
         }
@@ -216,7 +217,7 @@ process_service_list (xmlNode           *element,
                       const char        *udn,
                       const char        *service_type,
                       const char        *description_url,
-                      SoupUri           *url_base)
+                      SoupURI           *url_base)
 {
         for (element = element->children; element; element = element->next) {
                 xmlChar *prop;
@@ -275,7 +276,7 @@ process_device_list (xmlNode           *element,
                      const char        *udn,
                      const char        *service_type,
                      const char        *description_url,
-                     SoupUri           *url_base)
+                     SoupURI           *url_base)
 {
         for (element = element->children; element; element = element->next) {
                 xmlNode *children;
@@ -359,9 +360,9 @@ process_device_list (xmlNode           *element,
         }
 }
 
-/**
+/*
  * Called when the description document is loaded.
- **/
+ */
 static void
 description_loaded (GUPnPControlPoint *control_point,
                     XmlDocWrapper     *doc,
@@ -370,7 +371,7 @@ description_loaded (GUPnPControlPoint *control_point,
                     const char        *description_url)
 {
         xmlNode *element;
-        SoupUri *url_base;
+        SoupURI *url_base;
 
         /* Save the URL base, if any */
         element = xml_util_get_element ((xmlNode *) doc->doc,
@@ -396,11 +397,12 @@ description_loaded (GUPnPControlPoint *control_point,
         soup_uri_free (url_base);
 }
 
-/**
+/*
  * Description URL downloaded.
- **/
+ */
 static void
-got_description_url (SoupMessage           *msg,
+got_description_url (SoupSession           *session,
+                     SoupMessage           *msg,
                      GetDescriptionURLData *data)
 {
         XmlDocWrapper *doc;
@@ -430,8 +432,8 @@ got_description_url (SoupMessage           *msg,
                 xmlDoc *xml_doc;
 
                 /* Parse response */
-                xml_doc = xmlParseMemory (msg->response.body,
-                                          msg->response.length);
+                xml_doc = xmlParseMemory (msg->response_body->data,
+                                          msg->response_body->length);
                 if (xml_doc) {
                         doc = xml_doc_wrapper_new (xml_doc);
 
@@ -464,14 +466,14 @@ got_description_url (SoupMessage           *msg,
         get_description_url_data_free (data);
 }
 
-/**
+/*
  * Downloads and parses (or takes from cache) @description_url,
  * creating:
  *  - A #GUPnPDeviceProxy for the device specified by @udn if @service_type
- *    is NULL.
+ *    is %NULL.
  *  - A #GUPnPServiceProxy for the service of type @service_type from the device
- *    specified by @udn if @service_type is not NULL.
- **/
+ *    specified by @udn if @service_type is not %NULL.
+ */
 static void
 load_description (GUPnPControlPoint *control_point,
                   const char        *description_url,
@@ -512,7 +514,8 @@ load_description (GUPnPControlPoint *control_point,
                         return;
                 }
 
-                message_set_user_agent (data->message);
+                http_request_set_user_agent (data->message);
+                http_request_set_accept_language (data->message);
 
                 data->control_point   = control_point;
 
@@ -526,7 +529,7 @@ load_description (GUPnPControlPoint *control_point,
 
 	        soup_session_queue_message (session,
                                             data->message,
-                                            (SoupMessageCallbackFn)
+                                            (SoupSessionCallback)
                                                    got_description_url,
                                             data);
         }
@@ -901,6 +904,12 @@ gupnp_control_point_class_init (GUPnPControlPointClass *klass)
  * @context: A #GUPnPContext
  * @target: The search target
  *
+ * Create a new #GUPnPControlPoint with the specified @context and @target.
+ *
+ * @target should be a service or device name, such as
+ * <literal>urn:schemas-upnp-org:service:WANIPConnection:1</literal> or
+ * <literal>urn:schemas-upnp-org:device:MediaRenderer:1</literal>.
+ *
  * Return value: A new #GUPnPControlPoint object.
  **/
 GUPnPControlPoint *
@@ -922,6 +931,13 @@ gupnp_control_point_new (GUPnPContext *context,
  * @factory: A #GUPnPResourceFactory
  * @target: The search target
  *
+ * Create a new #GUPnPControlPoint with the specified @context, @factory and
+ * @target.
+ *
+ * @target should be a service or device name, such as
+ * <literal>urn:schemas-upnp-org:service:WANIPConnection:1</literal> or
+ * <literal>urn:schemas-upnp-org:device:MediaRenderer:1</literal>.
+ *
  * Return value: A new #GUPnPControlPoint object.
  **/
 GUPnPControlPoint *
@@ -931,6 +947,7 @@ gupnp_control_point_new_full (GUPnPContext         *context,
 {
         g_return_val_if_fail (GUPNP_IS_CONTEXT (context), NULL);
         g_return_val_if_fail (GUPNP_IS_RESOURCE_FACTORY (factory), NULL);
+        g_return_val_if_fail (target, NULL);
 
         return g_object_new (GUPNP_TYPE_CONTROL_POINT,
                              "client", context,
@@ -943,7 +960,9 @@ gupnp_control_point_new_full (GUPnPContext         *context,
  * gupnp_control_point_get_context
  * @control_point: A #GUPnPControlPoint
  *
- * Return value: The #GUPnPContext associated with @control_point.
+ * Get the #GUPnPControlPoint associated with @control_point.
+ *
+ * Return value: The #GUPnPContext.
  **/
 GUPnPContext *
 gupnp_control_point_get_context (GUPnPControlPoint *control_point)
@@ -962,8 +981,10 @@ gupnp_control_point_get_context (GUPnPControlPoint *control_point)
  * gupnp_control_point_list_device_proxies
  * @control_point: A #GUPnPControlPoint
  *
- * Return value: A #GList of discovered #GUPnPDeviceProxy objects. Do not
- * free the list nor its elements.
+ * Get the #GList of discovered #GUPnPDeviceProxy objects. Do not free the list
+ * nor its elements.
+ *
+ * Return value: a #GList of #GUPnPDeviceProxy objects.
  **/
 const GList *
 gupnp_control_point_list_device_proxies (GUPnPControlPoint *control_point)
@@ -977,8 +998,10 @@ gupnp_control_point_list_device_proxies (GUPnPControlPoint *control_point)
  * gupnp_control_point_list_service_proxies
  * @control_point: A #GUPnPControlPoint
  *
- * Return value: A #GList of discovered #GUPnPServiceProxy objects. Do not
- * free the list nor its elements.
+ * Get the #GList of discovered #GUPnPServiceProxy objects. Do not free the list
+ * nor its elements.
+ *
+ * Return value: a #GList of #GUPnPServiceProxy objects.
  **/
 const GList *
 gupnp_control_point_list_service_proxies (GUPnPControlPoint *control_point)
@@ -992,7 +1015,9 @@ gupnp_control_point_list_service_proxies (GUPnPControlPoint *control_point)
  * gupnp_control_point_get_resource_factory
  * @control_point: A #GUPnPControlPoint
  *
- * Return value: The #GUPnPResourceFactory used by the @control_point.
+ * Get the #GUPnPResourceFactory used by the @control_point.
+ *
+ * Return value: A #GUPnPResourceFactory.
  **/
 GUPnPResourceFactory *
 gupnp_control_point_get_resource_factory (GUPnPControlPoint *control_point)
