@@ -28,6 +28,7 @@
 
 #include "gupnp-context.h"
 #include "gupnp-service-info.h"
+#include "gupnp-resource-factory.h"
 
 G_BEGIN_DECLS
 
@@ -57,6 +58,12 @@ gupnp_device_info_get_type (void) G_GNUC_CONST;
 
 typedef struct _GUPnPDeviceInfoPrivate GUPnPDeviceInfoPrivate;
 
+/**
+ * GUPnPDeviceInfo:
+ *
+ * This struct contains private data only, and should be accessed using the
+ * functions below.
+ */
 typedef struct {
         GObject parent;
 
@@ -86,7 +93,7 @@ gupnp_device_info_get_context           (GUPnPDeviceInfo *info);
 const char *
 gupnp_device_info_get_location          (GUPnPDeviceInfo *info);
 
-const SoupUri *
+const SoupURI *
 gupnp_device_info_get_url_base          (GUPnPDeviceInfo *info);
 
 const char *
@@ -134,6 +141,9 @@ gupnp_device_info_get_icon_url          (GUPnPDeviceInfo *info,
                                          int             *width,
                                          int             *height);
 
+char *
+gupnp_device_info_get_presentation_url  (GUPnPDeviceInfo *info);
+
 GList *
 gupnp_device_info_list_devices          (GUPnPDeviceInfo *info);
 
@@ -154,6 +164,9 @@ GUPnPServiceInfo *
 gupnp_device_info_get_service           (GUPnPDeviceInfo *info,
                                          const char      *type);
 
+GUPnPResourceFactory *
+gupnp_device_info_get_resource_factory  (GUPnPDeviceInfo *device_info);
+
 G_END_DECLS
 
 #endif /* __GUPNP_DEVICE_INFO_H__ */
