@@ -518,7 +518,7 @@ gupnp_service_info_get_control_url (GUPnPServiceInfo *info)
  * gupnp_service_info_get_event_subscription_url
  * @info: A #GUPnPServiceInfo
  *
- * Get the event subscription URL for this service, or NULL.
+ * Get the event subscription URL for this service, or %NULL.
  *
  * Return value: A string. This string should be freed with g_free() after use.
  **/
@@ -535,13 +535,13 @@ gupnp_service_info_get_event_subscription_url (GUPnPServiceInfo *info)
 /**
  * gupnp_service_info_get_introspection
  * @info: A #GUPnPServiceInfo
- * @error: return location for a GError, or NULL
+ * @error: return location for a #GError, or %NULL
  *
  * Note that introspection object is created from the information in service
  * description document (SCPD) provided by the service so it can not be created
  * if the service does not provide an SCPD.
  *
- * Return value: A new #GUPnPServiceIntrospection for this service or NULL.
+ * Return value: A new #GUPnPServiceIntrospection for this service or %NULL.
  * Unref after use.
  **/
 GUPnPServiceIntrospection *
@@ -577,7 +577,7 @@ gupnp_service_info_get_introspection (GUPnPServiceInfo *info,
                 return NULL;
         }
 
-        message_set_user_agent (msg);
+        http_request_set_user_agent (msg);
 
         /* Send off the message */
         session = _gupnp_context_get_session (info->priv->context);
@@ -612,9 +612,9 @@ gupnp_service_info_get_introspection (GUPnPServiceInfo *info,
         return introspection;
 }
 
-/**
+/*
  * SCPD URL downloaded.
- **/
+ */
 static void
 got_scpd_url (SoupSession    *session,
               SoupMessage    *msg,
@@ -714,7 +714,7 @@ gupnp_service_info_get_introspection_async
                 return;
         }
 
-        message_set_user_agent (data->message);
+        http_request_set_user_agent (data->message);
 
         data->info      = info;
         data->callback  = callback;
